# -*- coding: utf_8 -*-

from pandac.PandaModules import *
from direct.showbase import DirectObject
from direct.task import Task


class gameTank(object):
	Zold = 0
	a = 0
	Qold = Quat()
	AnglePold = 0
	speed = 0.08
	def __init__(self, model, pos, nick, mask):
		
		# ключи для управления через клаву
		self.keyMap = {	"norma":0,"left":0, "right":0, "forward":0, "back":0, 
								"bashnya-left":0, "bashnya-right":0, "cam-up":0,
								"cam-down":0, "fire":0, "cam-left":0, "cam-right":0}
								

		self.initCollision()
		self.loadModel(model, pos)
		self.loadLabel(nick)
		self.setUpCamera()
		self.createCollisions(mask)
		self.attachControls()
		taskMgr.add(self.move, 'move-task-PLAYER')

	def initCollision(self):
		""" create the collision system """
		base.cTrav = CollisionTraverser()
		#self.pusher = CollisionHandlerPusher()

	def loadModel(self, model, pos):
		tX,tY,tZ,tH,tP,tR=pos
		# пустая нода игрока для управления
		self.node = NodePath('player')
		self.node.reparentTo(render)
		self.node.setScale(1)
		#устанавливаем маску для коллизии
		self.node.setCollideMask(BitMask32(2))
		# модель танка
		self.model=loader.loadModel(model)
		self.model.reparentTo(self.node)
		self.model.setScale(0.0003,0.0003,0.0003)
		
		self.node.setPos(tX,tY,tZ)
		self.node.setHpr(tH,tP,tR)
		
		# разносим танк на детали
		self.stvol = self.model.find('**/**/stvol')
		self.bashnya = self.model.find('**/**/bashnya')
		self.a = self.node.getH()
		self.Zold = self.node.getZ()
		
	def loadLabel(self, nick):
		#добавляем текст над танком
		font = loader.loadFont('arial.ttf')
		
		newTextNode = TextNode('text')
		newTextNode.setFont(font)
		newTextNode.setAlign(TextNode.ACenter)
		newTextNode.setText(nick)
		newTextNode.setWordwrap(16.0)
		
		newTextNode.setShadow(0.05,0.05)
		newTextNode.setShadowColor(1, 1, 1, 0.5)
		
		newTextNode.setTextColor(0,0,0,1)
		
		newTextNode.setFrameColor(0, 0, 1, 0.3)
		newTextNode.setFrameAsMargin(0.3, 0.3, 0.3, 0.3)
		
		newTextNode.setCardColor(1, 0, 0, 0.7)
		newTextNode.setCardAsMargin(0.3, 0.3, 0.3, 0.3)
		newTextNode.setCardDecal(True)
		
		text_generate = newTextNode.generate()
		
		self.newTextNodePath = render.attachNewNode(text_generate)
		self.newTextNodePath.setScale(0.08)
		self.newTextNodePath.reparentTo(self.node)
		self.newTextNodePath.setZ(0.9)
		self.newTextNodePath.setBillboardPointEye()

	def setUpCamera(self):
		# отключаем стандартное управление мышой
		base.disableMouse()
		# определяем водителя для камеры		
		self.boxCam = render.attachNewNode('boxCam') 
		self.boxCam.setPos(self.node.getPos())
		self.boxCam.setZ(self.node.getZ()+1)
		self.boxCam.setHpr(self.node.getHpr())
		
		# устанавлеваем начальное положение камеры
		base.camera.setPos(self.boxCam.getX(), self.boxCam.getY()-7, 1)
		base.camera.reparentTo(self.boxCam)
		base.camera.lookAt(self.boxCam)

	def createCollisions(self, mask):
		
		# луч для коллизии с полом и нахождения нормали к поверхности
		# 1
		self.tankGroundCol1 = CollisionNode('tankRay')
		self.tankGroundCol1.addSolid(CollisionRay(0,0,1,0,0,-1))
		self.tankGroundCol1.setFromCollideMask(BitMask32.bit(mask))
		self.tankGroundCol1.setIntoCollideMask(BitMask32.allOff())
		self.tankGroundColNp1 = self.node.attachNewNode(self.tankGroundCol1)
		self.tankGroundHandler1 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.tankGroundColNp1, self.tankGroundHandler1)
		
		
		# четыре шара по углам танка для коллиции с препядствиями
		# 1
		self.EnvCol1 = CollisionNode('colEnv_1')
		self.EnvCol1.addSolid(CollisionSphere(-0.25, 0.55, 0.5, 0.3))
		
		#self.EnvCol1.setIntoCollideMask(BitMask32.bit(1))
		self.EnvCol1.setFromCollideMask(BitMask32.bit(1))
		self.EnvCol1.setIntoCollideMask(BitMask32.allOff())
		#self.EnvCol1.setFromCollideMask(BitMask32.allOff())
		
		self.EnvColNp1 = self.node.attachNewNode(self.EnvCol1)
		self.pusher1 = CollisionHandlerPusher()
		self.pusher1.addCollider(self.EnvColNp1, self.node)
		base.cTrav.addCollider(self.EnvColNp1, self.pusher1)
		
		# 2
		self.EnvCol2 = CollisionNode('colEnv_2')
		self.EnvCol2.addSolid(CollisionSphere(0.25, 0.55, 0.5, 0.3))
		
		#self.EnvCol2.setIntoCollideMask(BitMask32.bit(1))
		self.EnvCol2.setFromCollideMask(BitMask32.bit(1))
		self.EnvCol2.setIntoCollideMask(BitMask32.allOff())
		#self.EnvCol2.setFromCollideMask(BitMask32.allOff())
		
		self.EnvColNp2 = self.node.attachNewNode(self.EnvCol2)
		self.pusher2 = CollisionHandlerPusher()
		self.pusher2.addCollider(self.EnvColNp2, self.node)
		base.cTrav.addCollider(self.EnvColNp2, self.pusher2)
		
		# 3
		self.EnvCol3 = CollisionNode('colEnv_3')
		self.EnvCol3.addSolid(CollisionSphere(-0.25, -0.55, 0.5, 0.3))
		
		#self.EnvCol3.setIntoCollideMask(BitMask32.bit(1))
		self.EnvCol3.setFromCollideMask(BitMask32.bit(1))
		self.EnvCol3.setIntoCollideMask(BitMask32.allOff())
		#self.EnvCol3.setFromCollideMask(BitMask32.allOff())
		
		self.EnvColNp3 = self.node.attachNewNode(self.EnvCol3)
		self.pusher3 = CollisionHandlerPusher()
		self.pusher3.addCollider(self.EnvColNp3, self.node)
		base.cTrav.addCollider(self.EnvColNp3, self.pusher3)

		# 4
		self.EnvCol4 = CollisionNode('colEnv_3')
		self.EnvCol4.addSolid(CollisionSphere(0.25, -0.55, 0.5, 0.3))
		
		#self.EnvCol4.setIntoCollideMask(BitMask32.bit(1))
		self.EnvCol4.setFromCollideMask(BitMask32.bit(1))
		self.EnvCol4.setIntoCollideMask(BitMask32.allOff())
		#self.EnvCol4.setFromCollideMask(BitMask32.allOff())
		
		self.EnvColNp4 = self.node.attachNewNode(self.EnvCol4)
		self.pusher4 = CollisionHandlerPusher()
		self.pusher4.addCollider(self.EnvColNp4, self.node)
		base.cTrav.addCollider(self.EnvColNp4, self.pusher4)
		
		#"""
		self.tankGroundColNp1.show()
		self.EnvColNp1.show()
		self.EnvColNp2.show()
		self.EnvColNp3.show()
		self.EnvColNp4.show()
		#"""

	# обработчик нажатия клавиши
	def setKey(self, key, value):
		self.keyMap[key] = value
		
	def attachControls(self):
		# связываем клавиши с действиями
		base.accept("arrow_left", self.setKey, ["left",1])
		base.accept("arrow_right", self.setKey, ["right",1])
		base.accept("arrow_up", self.setKey, ["forward",1])
		base.accept("arrow_down", self.setKey, ["back",1])
		base.accept("a", self.setKey, ["bashnya-left",1])
		base.accept("s", self.setKey, ["bashnya-right",1])
		
		base.accept("q", self.setKey, ["cam-left",1])
		base.accept("w", self.setKey, ["cam-right",1])
		
		base.accept("space", self.setKey, ["fire",1])
		base.accept("space-up", self.setKey, ["fire",0])
		
		base.accept("n", self.setKey, ["norma",1])
		base.accept("n-up", self.setKey, ["norma",0])
		
		base.accept("page_up", self.setKey, ["cam-up",1])
		base.accept("page_up-up", self.setKey, ["cam-up",0])
		
		base.accept("page_down", self.setKey, ["cam-down",1])
		base.accept("page_down-up", self.setKey, ["cam-down",0])
		
		base.accept("arrow_left-up", self.setKey, ["left",0])
		base.accept("arrow_right-up", self.setKey, ["right",0])
		base.accept("arrow_up-up", self.setKey, ["forward",0])
		base.accept("arrow_down-up", self.setKey, ["back",0])
		base.accept("a-up", self.setKey, ["bashnya-left",0])
		base.accept("s-up", self.setKey, ["bashnya-right",0])
		
		base.accept("q-up", self.setKey, ["cam-left",0])
		base.accept("w-up", self.setKey, ["cam-right",0])

	def move(self, task):
		elapsed = globalClock.getDt()
		
		# поворот камеры по горизонту
		if (self.keyMap["cam-left"]!=0):
			base.camera.setX(base.camera, +(elapsed*5.0))
		if (self.keyMap["cam-right"]!=0):
			base.camera.setX(base.camera, -(elapsed*5.0))
		
		# поворот камеры по вертикали
		if (self.keyMap["cam-up"]!=0):
			base.camera.setZ(base.camera, +(elapsed*5.0))
		if (self.keyMap["cam-down"]!=0):
			base.camera.setZ(base.camera, -(elapsed*5.0))
		
		# анимация выстрела
		if (self.keyMap["fire"]!=0):
			povorotS = self.stvol.getHpr()
			self.stvol.setHpr(0, 6, 0)
			self.keyMap["fire"]=0
		if (self.keyMap["fire"]==0):
			povorotS = self.stvol.getHpr()
			if povorotS[1]>0:
				self.stvol.setHpr(0, povorotS[1]-.5, 0)
		
		# поворот башни
		povorotB = self.bashnya.getHpr()
		if (self.keyMap["bashnya-left"]!=0):
			self.bashnya.setH(self.bashnya, +(elapsed*60.0))
		if (self.keyMap["bashnya-right"]!=0):
			self.bashnya.setH(self.bashnya, -(elapsed*60.0))
		
		# поворот танка
		if (self.keyMap["left"]!=0):
			self.a = self.a + elapsed*75.0
		#else:
			#self.a = self.AnglePold
		if (self.keyMap["right"]!=0):
			self.a = self.a - elapsed*75.0
		#else:
			#self.a = self.AnglePold
		
		# движение танка
		if (self.keyMap["forward"]!=0):
			self.node.setY(self.node, self.speed)
		if (self.keyMap["back"]!=0):
			self.node.setY(self.node, -(self.speed/2.0))
			
		# возврат в исходное состояние камеры? и башни
		if (self.keyMap["norma"]!=0):
			self.bashnya.setHpr(0, 0, 0)
			self.node.setPos(0,0,50)
		#####################################################################
		startpos = self.node.getPos()
		####################################################################################
		# вычисление высоты танка по поверхности 1
		entries = []
		for i in range(self.tankGroundHandler1.getNumEntries()):
			entry = self.tankGroundHandler1.getEntry(i)
			entries.append(entry)
			
		entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),x.getSurfacePoint(render).getZ()))
		
		if (len(entries)>0):
			# смягчаем прыжок по Z тут какраз и нужен тег который мы задали в начале
			Znew=entries[0].getSurfacePoint(render).getZ()
			
			deltaZ = (Znew - self.Zold)/1.2
			self.node.setZ(Znew-deltaZ)
			
			if (self.Zold - Znew)>0.1:
				self.node.setY(self.node, elapsed*2.2)
			
			if (self.Zold - Znew)>0.08:
				self.EnvColNp1.setZ(1)
				self.EnvColNp2.setZ(1)
				self.EnvColNp3.setZ(1)
				self.EnvColNp4.setZ(1)
			else:
				self.EnvColNp1.setZ(0)
				self.EnvColNp2.setZ(0)
				self.EnvColNp3.setZ(0)
				self.EnvColNp4.setZ(0)
			
			self.Zold = self.node.getZ()
			
		else:
			self.node.setPos(startpos)
		
		# самое интересное попытка наклонять танк повторяя поверхность по которой он движется - в разработке
		if entries:
			#вылетает ошибка по поводу коллизий буду решать потом....
			#try:
			angleH=(180*entries[0].getInto().getNormal().getX())/3.1415
			angleP=(180*entries[0].getInto().getNormal().getY())/3.1415
			angleR=-(180*(entries[0].getInto().getNormal().getZ()-1))/3.1415
		
			#
			n = entries[0].getInto().getNormal()
			q = Quat()
			q.setFromAxisAngle(self.a,n)
			povorot = self.Qold - (self.Qold - q)/5
			povorotP = self.AnglePold - (self.AnglePold - angleP)/5
			self.node.setQuat(povorot)
			self.node.setP(self.node, -povorotP)
			#
			self.Qold = povorot
			self.AnglePold = povorotP
			#except:
				#pass
			
		#######################################################################
		
		# двигаем boxCam за танком
		self.boxCam.setPos(self.node.getPos())
		self.boxCam.setZ(self.node.getZ()+1)
		self.boxCam.setH(self.node.getH())
		base.camera.reparentTo(self.boxCam)
		base.camera.lookAt(self.boxCam)
		#
		#base.cTrav.traverse(render)
		
		#выходим из обработчика
		return Task.cont